<?php

/*
 * phpWebFileManager - simple file management PHP script
 *
 * index.php - main script file
 * ____________________________________________________________
 *
 * Developed by Ondrej Jombik <nepto@pobox.sk>
 * Copyright (c) 2001-2002 Platon SDG, http://www.platon.sk/
 * All rights reserved.
 *
 * See README file for more information about this software.
 * See COPYING file for license information.
 *
 * Download the latest version from
 * http://www.platon.sk/projects/phpWebFileManager/
 */

/* $Id: index.php,v 1.30 2002/08/04 23:22:58 nepto Exp $ */

/*
 * phpWebFileManager initialization
 *
 * Config file is read from init.inc.php file.
 */

$fm_init_file = dirname(__FILE__)
	. (strlen(dirname(__FILE__)) > 0 ? '/' : '')
	. 'init.inc.php';

	if (! @file_exists($fm_init_file)) {
		exit;
	}

require_once $fm_init_file;

/*
 * Libraries function inclusion
 */

require_once $PN_PathPrefix . 'lib/fnc.append_slash.php';
require_once $PN_PathPrefix . 'lib/fnc.get_path_without_scriptname.php';
require_once $PN_PathPrefix . 'lib/fnc.glue_url.php';
require_once $PN_PathPrefix . 'lib/fnc.write_origvars_html.php';

/*
 * fm_echo_message()
 *
 * Writes message on output.
 */

function fm_echo_message($message)
{
	echo "<small>$message</small>";
}

function fm_get_cgi_var($name, $default_value = null)
{
	global $fm_cfg;
	$name = $fm_cfg['cgi'].$name;

	global $HTTP_GET_VARS;
	$var = $HTTP_GET_VARS[$name];

	if (! isset($var)) {
		global $HTTP_POST_VARS;
		$var = $HTTP_POST_VARS[$name];
	}

	if (isset($var)) {
		$var = stripslashes($var);
	} else {
		$var = $default_value;
	}

	return $var;
}


/*
 * Main phpWebFileManager function
 */

function fm_main()
{
	global $fm_cfg;
	global $HTTP_SERVER_VARS;

	/* Hidden parameters */
	$fm_origvars            = append_slash($fm_cfg['origvars'], '&');
	$fm_cfg['dir']['root']  = append_slash($fm_cfg['dir']['root']);
	$fm_cfg['url']['root']  = append_slash($fm_cfg['url']['root']);
	$fm_cfg['url']['icons'] = append_slash($fm_cfg['url']['icons']);
	if (strlen($fm_cfg['format']['date']) <= 0) {
		$fm_cfg['format']['date'] = _FM_DATE_FORMAT;
	}


	/*
	 * Getting CGI variables
	 */
	$fm_action       = fm_get_cgi_var('action',       '');
	$fm_submit       = fm_get_cgi_var('submit',       '');
	$fm_dir          = fm_get_cgi_var('dir',          '');
	$fm_filename     = fm_get_cgi_var('filename',     '');
	$fm_new_filename = fm_get_cgi_var('new_filename', '');
	$fm_file_data    = fm_get_cgi_var('file_data',    '');
	$PHP_SELF        = $HTTP_SERVER_VARS['PHP_SELF'];

	/*
	 * Debugging CGI varaibles
	 */
	if (0) {
		echo '<hr><pre>';
		foreach (array('fm_action', 'fm_submit', 'fm_dir', 'fm_filename', 'fm_new_filename', 'fm_file_data') as $var_name) {
			echo htmlspecialchars(str_pad($var_name, 15).' = '.$$var_name)."\n";
		}
		echo '</pre><hr>';
	}

	/*
	 * Symlink detection (must be before following directory modifying)
	 */

	if (! $fm_cfg['perm']['dir']['follow_symlinks']) {
		$fm_test_symlink = $fm_dir;

		while ($fm_test_symlink != '') {
			if ((@file_exists($fm_test_symlink) && @is_link($fm_test_symlink))
					|| (@file_exists($fm_test_symlink) && @is_link("$fm_test_symlink/"))) {
				$fm_dir = $fm_cfg['dir']['start'];
				break;
			}

			$fm_test_symlink = substr($fm_test_symlink,
					0, intval(strrpos($fm_test_symlink, '/')));
		}
	}

	if (! isset($fm_dir) || ! $fm_cfg['perm']['dir']['enter']) {
		$fm_dir = $fm_cfg['dir']['start'];
	}

	/* 
	 * Directory modifying. Dirty, but safe.
	 */

	$fm_dir = preg_replace('/\/\/+/','/', "$fm_dir/");
	$fm_dir = preg_replace('/^(|.*\/)\.\.\/([^\.\/][^\.\/]|[^\/][^\.\/]|[^\/][^\.\/]|[^\/]{3,})\/(.*)$/', '\\1\\3', $fm_dir);
	$fm_dir = preg_replace('/^(|.*\/)([^\/\.][^\.\/]|[^\/][^\.\/]|[^\/][^\.\/]|[^\/]{3,})\/\.\.\/(.*)$/', '\\1\\3', $fm_dir);

	while (preg_match('/^(|.*\/)\.\.\/(.*)$/', $fm_dir)) {
		$fm_dir = preg_replace('/^(|.*\/)\.\.\/(.*)$/', '\\1\\2', $fm_dir);
	}
	while (strlen($fm_dir) > 0 && $fm_dir[0] == '/') {
		$fm_dir = substr($fm_dir, 1, strlen($fm_dir));
	}

	/*
	 * Change to root directory
	 */

	/* Store the starting working directory. */
	$fm_orig_dir = getcwd();

	if (@is_dir($fm_cfg['dir']['root'])) {
		chdir($fm_cfg['dir']['root']);
	}

	$fm_root_dir = getcwd();

	if (@file_exists($fm_dir) && @is_dir($fm_dir)) {
		chdir($fm_dir);

		/* On some *BSD systems with PHP version <= 4.0.6
		   function chdir() is removing last '/' character
		   in $fm_dir variable, so we must improve it. */
		$fm_dir = append_slash($fm_dir);
	} else  {
		$fm_dir = '';
	}

	/*
	 * Prepare URL & HTML encode of some variables.
	 */

	$fm_filename_he = htmlentities($fm_filename);
	$fm_filename_he2 = str_replace(' ', '&nbsp;', $fm_filename_he);
	//$fm_filename_ue = htmlentities(rawurlencode($fm_filename));
	$fm_dir_he = htmlentities($fm_dir);
	$fm_dir_he2 = str_replace(' ', '&nbsp;', $fm_dir_he);
	$fm_dir_ue = htmlentities(rawurlencode($fm_dir));
	$PHP_SELF_origvars_ue = htmlentities("$PHP_SELF?$fm_origvars");

	/*
	 * Title writting with current working direcotry information.
	 */

	if (defined('LOADED_AS_MODULE')) {
		OpenTable();
		echo '<font class="pn-title">'._FM_FILE_MANAGER
			. (strlen($fm_dir) > 0 ? ": &nbsp;$fm_dir_he" : '') 
			. '</font>';
		CloseTable();
		OpenTable();
	}
	else {
		if (strlen($fm_dir) > 0)
			echo "<h3>$fm_dir_he2</h3>\n";
	}

	/*
	 * Form
	 */

	echo '<form enctype="multipart/form-data" method=post'
		. ' action="' . htmlentities($PHP_SELF) . '">'."\n";

	write_origvars_html($fm_origvars);

	echo '<input type=hidden value="'.$fm_filename_he.'" name="'.$fm_cfg['cgi'].'filename">'."\n";
	echo '<input type=hidden value="'.$fm_dir_he.'" name="'.$fm_cfg['cgi'].'dir">'."\n";

	/* PN: PostNuke module calls */
	if (defined('LOADED_AS_MODULE')) {
		echo '<input type=hidden name="op" value="modload">'."\n";
		echo '<input type=hidden name="name" value="'.$ModName.'">'."\n";
		echo '<input type=hidden name="file" value="index">'."\n";
	}

	switch ($fm_action) {

		/*
		 * confirm delete files action
		 */
		case 'confirm_delete_file':
			echo '<input type=hidden value=delete_file name="'.$fm_cfg['cgi'].'action">';

			echo _FM_REALLY_DELETE.'<br><br><i>'.$fm_filename_he2.'</i><br><br>';
			echo $fm_dir == ''
				? _FM_FROM_ROOT_DIR.'?<br><br>'
				: _FM_FROM_DIR.'<br><br><i>'.$fm_dir_he2.'</i><br><br>';
			echo '<input type=submit name="'.$fm_cfg['cgi'].'submit" value="'._FM_FILE_DELETE.'">&nbsp;';
			echo '<input type=submit name="'.$fm_cfg['cgi'].'submit" value="'._FM_CANCEL.'">';
			break;

			/*
			 * confirm remove directory action
			 */
		case 'confirm_remove_directory':
			echo '<input type=hidden value=remove_directory name="'.$fm_cfg['cgi'].'action">';

			echo _FM_REALLY_REMOVE.'<br><br><i>'.$fm_filename_he2.'</i><br><br>';
			echo _FM_MUST_BE_EMPTY.'<br><br>';
			echo '<input type=submit name="'.$fm_cfg['cgi'].'submit" value="'._FM_DIR_REMOVE.'">&nbsp;';
			echo '<input type=submit name="'.$fm_cfg['cgi'].'submit" value="'._FM_CANCEL.'">';
			break;

			/*
			 * confirm rename files action and confirm rename directory action
			 */
		case 'confirm_rename_file':
		case 'confirm_rename_directory':
			echo '<input type=hidden value="';
			echo $fm_action == 'confirm_rename_directory' ? 'rename_directory' : 'rename_file';
			echo '" name="'.$fm_cfg['cgi'].'action">';

			echo _FM_RENAME_FROM.'<br><br><i>'.$fm_filename_he2.'</i><br><br>';
			echo _FM_RENAME_TO.'<br><br>';
			echo '<input type=text value="" name="'.$fm_cfg['cgi'].'new_filename"><br><br>';
			echo '<input type=submit name="'.$fm_cfg['cgi'].'submit" value="';
			echo $fm_action == 'confirm_rename_directory' ? _FM_DIR_RENAME : _FM_FILE_RENAME;
			echo '">&nbsp;';
			echo '<input type=submit name="'.$fm_cfg['cgi'].'submit" value="'._FM_CANCEL.'">';
			break;

			/*
			 * confirm create directory and confirm create file actions
			 */
		case 'confirm_create_directory':
		case 'confirm_create_file':
			echo '<input type=hidden value="';
			echo $fm_action == 'confirm_create_directory' ? 'create_directory' : 'create_file';
			echo '" name="'.$fm_cfg['cgi'].'action">';

			echo _FM_ENTER_NAME . ':<br><br>';
			echo '<input type=text value="" name="'.$fm_cfg['cgi'].'new_filename"><br><br>';
			echo '<input type=submit name="'.$fm_cfg['cgi'].'submit" value="';
			echo $fm_action == 'confirm_create_directory' ? _FM_DIR_CREATE : _FM_FILE_CREATE;
			echo '">&nbsp;';
			echo '<input type=submit name="'.$fm_cfg['cgi'].'submit" value="'._FM_CANCEL.'">';
			break;

			/*
			 * confirm upload file action
			 */
		case 'confirm_upload_file':
			echo '<input type=hidden value="upload_file" name="'.$fm_cfg['cgi'].'action">';

			echo _FM_SELECT_LOCAL . ':<br><br>';
			echo '<input type=file value="" name="'.$fm_cfg['cgi'].'userfile"><br><br>';
			echo '<input type=submit name="'.$fm_cfg['cgi'].'submit" value="'._FM_FILE_UPLOAD.'">&nbsp;';
			echo '<input type=submit name="'.$fm_cfg['cgi'].'submit" value="'._FM_CANCEL.'">';
			break;

			/*
			 * confirm edit file and edit file form
			 */
		case 'confirm_edit_file':
			if (is_readable($fm_filename) && is_writeable($fm_filename)) {
				// TODO:
				// 1. check file type - if binary, ask if really edit
				// 2. check file size - if too big, ask if really edit
				echo '<input type=hidden value="edit_file" name="'.$fm_cfg['cgi'].'action">';

				echo _FM_REALLY_EDIT
					. '<br><br><i>'.$fm_filename_he2.'</i><br><br>';
				echo $fm_dir == ''
					? _FM_IN_ROOT_DIR . '?<br><br>'
					: _FM_IN_DIR . '<br><br><i>'.$fm_dir_he2.'</i><br><br>';
				echo '<input type=submit name="'.$fm_cfg['cgi'].'submit" value="'._FM_FILE_EDIT.'">&nbsp;';
				echo '<input type=submit name="'.$fm_cfg['cgi'].'submit" value="'._FM_CANCEL.'">';
				break;
			}
		case 'edit_file':
			if (is_readable($fm_filename) && is_writeable($fm_filename)) {
				if ($fm_submit == _FM_FILE_EDIT) {
					echo '<input type=hidden value="save_file" name="'.$fm_cfg['cgi'].'action">';
					// Please write me, if you want to have cols and rows configurable.
					echo '<textarea name="'.$fm_cfg['cgi'].'file_data" cols=50 rows=15>';
					htmlentities(@readfile($fm_filename));
					echo '</textarea><br><br>';
					echo '<input type=submit name="'.$fm_cfg['cgi'].'submit" value="'._FM_FILE_SAVE.'">&nbsp;';
					echo '<input type=submit name="'.$fm_cfg['cgi'].'submit" value="'._FM_CANCEL.'">';
					break;
				} else fm_echo_message(_FM_CANCELED);
			} else {
				fm_echo_message(_FM_FILE_EDIT_ERR);
			}

			echo '<br><br>';

		default:

			/*
			 * save action
			 */
			if ($fm_action == 'save_file' && $fm_cfg['perm']['file']['save']) {
				if ($fm_submit == _FM_FILE_SAVE) {
					if($fp = @fopen($fm_filename, 'w')) {
						flock($fp, 2);
						fwrite($fp, $fm_file_data, strlen($fm_file_data));
						flock($fp, 3);
						fclose($fp);
						fm_echo_message(_FM_FILE_SAVE_OK);
					} else {
						fm_echo_message(_FM_FILE_SAVE_ERR);
					}
				}
				else fm_echo_message(_FM_CANCELED);
				echo '<br><br>';
			}

			/*
			 * delete action
			 */
			if ($fm_action == 'delete_file' && $fm_cfg['perm']['file']['delete']) {
				if ($fm_submit == _FM_FILE_DELETE) {
					if (@unlink($fm_filename) == false)
						fm_echo_message(_FM_FILE_DELETE_ERR);
					else
						fm_echo_message(_FM_FILE_DELETE_OK);
				}
				else fm_echo_message(_FM_CANCELED);
				echo '<br><br>';
			}

			/*
			 * remove direcotry
			 */
			if ($fm_action == 'remove_directory' && $fm_cfg['perm']['dir']['remove']) {
				if ($fm_submit == _FM_DIR_REMOVE) {
					if (@rmdir($fm_filename) == 0)
						fm_echo_message(_FM_DIR_REMOVE_ERR);
					else
						fm_echo_message(_FM_DIR_REMOVE_OK);
				}
				else fm_echo_message(_FM_CANCELED);
				echo '<br><br>';
			}

			/*
			 * rename action
			 */
			if (($fm_action == 'rename_directory' && $fm_cfg['perm']['dir']['rename'])
					|| ($fm_action == 'rename_file' && $fm_cfg['perm']['file']['rename'])) {
				if ($fm_new_filename != '' &&
						(($fm_submit == _FM_DIR_RENAME && $fm_action == "rename_directory")
						 || ($fm_submit == _FM_FILE_RENAME && $fm_action == 'rename_file')
						)) {
					if (@rename($fm_filename, $fm_new_filename) == false)
						fm_echo_message(_FM_RENAME_ERR);
					else
						fm_echo_message(_FM_RENAME_OK);
				}
				else fm_echo_message(_FM_CANCELED);
				echo '<br><br>';
			}

			/*
			 * create directory and create file actions
			 */
			if (($fm_action == 'create_directory' && $fm_cfg['perm']['dir']['create']) 
					|| ($fm_action == 'create_file' && $fm_cfg['perm']['file']['create'])) {
				if ($fm_new_filename != '' &&
						(($fm_submit == _FM_DIR_CREATE && $fm_action == 'create_directory')
						 || ($fm_submit == _FM_FILE_CREATE && $fm_action == 'create_file'))
				   ) {
					if ($fm_action == 'create_directory') {
						if (@mkdir($fm_new_filename, $fm_cfg['mode']['dir']) == false) {
							fm_echo_message(_FM_DIR_CREATE_ERR);
						} else {
							fm_echo_message(_FM_DIR_CREATE_OK);
						}
					} else {
						if (file_exists($fm_new_filename)) 
							fm_echo_message(_FM_FILE_CREATE_ERR1);
						else {
							if (($fh = @fopen($fm_new_filename, 'w')) == false) {
								fm_echo_message(_FM_FILE_CREATE_ERR2);
							} else {
								fclose($fh);
								fm_echo_message(_FM_FILE_CREATE_OK);

								if ($fm_cfg['mode']['file'] != 0) {
									if (@chmod($fm_new_filename, $fm_cfg['mode']['file']) == false) {
										fm_echo_message(' '.  _FM_CHMOD_ERR);
									} else {
										fm_echo_message(' '.  _FM_CHMOD_OK);
									}
								}
							}
						}
					}
				}
				else fm_echo_message(_FM_CANCELED);
				echo '<br><br>';
			}

			/*
			 * file upload action
			 */
			if ($fm_action == 'upload_file' && $fm_cfg['perm']['file']['upload']) {
				if ($fm_submit == _FM_FILE_UPLOAD && $fm_userfile_name != '') {

					$tmp_target_filename = $target_filename = $fm_userfile_name;

					$i = 2;
					while (file_exists($tmp_target_filename)) {
						if ($i == 2) {
							fm_echo_message(_FM_FILENAME_CHANGED);
							echo '<br>';
						}
						$tmp_target_filename = $target_filename . '-' . $i;
						$i++;
					}

					$target_filename = $tmp_target_filename;

					//echo("fm_userfile = $fm_userfile<br>");
					//echo("fm_userfile_name = $fm_userfile_name<br>");
					//echo("target_filename = $target_filename<br>");

					if (@copy($fm_userfile, $target_filename)) {

						fm_echo_message(_FM_FILE_UPLOAD_OK);

						if ($fm_cfg['mode']['file'] != 0) {
							if (@chmod($target_filename, $fm_cfg['mode']['file']) == false) {
								fm_echo_message(' '._FM_CHMOD_ERR);
							} else {
								fm_echo_message(' '._FM_CHMOD_OK);
							}
						}
					}
					else 
						fm_echo_message(_FM_FILE_UPLOAD_ERR);
				}
				else fm_echo_message(_FM_CANCELED);

				echo '<br><br>';
			}

			/*
			 * Column display counting
			 */

			$rename_column = $fm_cfg['perm']['dir']['rename'] || $fm_cfg['perm']['file']['rename'];
			$delete_column = $fm_cfg['perm']['dir']['remove'] || $fm_cfg['perm']['file']['delete'];
			$view_column = $fm_cfg['perm']['dir']['enter']    || $fm_cfg['perm']['file']['view'];
			$edit_column = $fm_cfg['perm']['file']['edit'];
			$last_row = $fm_cfg['perm']['dir']['create'] 
				|| $fm_cfg['perm']['file']['create']
				|| $fm_cfg['perm']['file']['upload'];

			/*
			 * Colspan counting
			 */

			$colspan = 1;
			if ($fm_cfg['show']['icons'])	$colspan++;
			if ($fm_cfg['show']['size'])	$colspan++;
			if ($fm_cfg['show']['date'])	$colspan++;
			if ($rename_column)				$colspan++;
			if ($delete_column)				$colspan++;
			if ($view_column)				$colspan++;
			if ($edit_column)				$colspan++;

			/*
			 * Table tag and parent directory link
			 */

			echo '<table cellpadding="2" class=phpWebFileManager>' . "\n";

			$line = 0;
			if (strlen(getcwd()) > strlen($fm_root_dir)) {
				echo '<tr valign=middle';
				if (! empty($fm_cfg['color']['odd']))
					echo ' bgcolor="'.$fm_cfg['color']['odd'].'"';
				echo '>';

				if ($fm_cfg['show']['icons']) {
					echo '<td align=center>';
					echo '<a href="'.$PHP_SELF_origvars_ue.$fm_cfg['cgi'].'dir='.$fm_dir_ue.'..">';
					echo '<img alt="'._FM_BACK.'" border=0 src="'
						. $fm_cfg['url']['icons'].$fm_cfg['icons']['sys']['UP-DIR'].'"></a>';
					echo '</td>';
				}

				echo '<td width="100%" colspan="'.($colspan-1).'">';
				echo '<a href="'.$PHP_SELF_origvars_ue.$fm_cfg['cgi'].'dir='.$fm_dir_ue.'..">';
				echo '('._FM_PARENT_DIR.')</a>';
				echo '</td>';

				echo '</tr>'."\n";

				$line++;
			}

			/*
			 * Two steps for() loop.
			 * One for directories and one for files.
			 */
			for ($k = 0; $k < 2; $k++) {

				$d = dir('./');

				/* Order in alphabetical order. */
				$listing = array();
				while ($entry = $d->read()) 
					array_push($listing, $entry);
				sort($listing);

				while ($entry = array_shift($listing)) {
					$entry_he  = htmlentities($entry);
					$entry_he2 = str_replace(' ', '&nbsp;', $entry_he);
					$entry_ue  = htmlentities(rawurlencode($entry));

					// Dirty fastfix by Asbjrn Vrnes <avaernes@online.no>
					$entry_ue = str_replace('&ouml;', '%F6', $entry_ue);
					$entry_ue = str_replace('&aelig;', '%E6', $entry_ue);
					$entry_ue = str_replace('&oslash;', '%F8', $entry_ue);
					$entry_ue = str_replace('&aring;', '%E5', $entry_ue);
					$entry_ue = str_replace('&Aelig;', '%C6', $entry_ue);
					$entry_ue = str_replace('&Oslash;', '%D8', $entry_ue);
					$entry_ue = str_replace('&Aring;', '%C5', $entry_ue);

					if ($entry != '.' && $entry != '..' && 
							(($k == 0 && @is_dir($entry)) || ($k == 1 && ! @is_dir($entry)))) {

						echo '<tr valign=middle';
						$line++;
						if ($line % 2) {
							if (! empty($fm_cfg['color']['odd']))
								echo ' bgcolor="'.$fm_cfg['color']['odd'].'"';
						} else {
							if (! empty($fm_cfg['color']['even']))
								echo ' bgcolor="'.$fm_cfg['color']['even'].'"';
						}
						echo '>';

						if (@is_dir($entry)) {
							if ($fm_cfg['show']['icons']) {
								echo '<td align=center>';
								if (is_link($entry)) {
									echo '<b>~</b>';
								} else {
									if ($fm_cfg['perm']['dir']['enter'])  {
										echo '<a href="'.$PHP_SELF_origvars_ue;
										echo $fm_cfg['cgi'].'dir='.$fm_dir_ue.$entry_ue.'">';
									}

									echo '<img alt="'._FM_DIR.'" border=0 src="'
										. $fm_cfg['url']['icons'].$fm_cfg['icons']['sys']['DIR'].'">';

									if ($fm_cfg['perm']['dir']['enter']) 
										echo '</a>';
								}

								echo '</td>';
							}

							echo '<td width="100%">';

							if ($fm_cfg['perm']['dir']['enter']
									&& ($fm_cfg['perm']['dir']['follow_symlinks']
										|| ! is_link($entry))) {
								echo '<a href="'.$PHP_SELF_origvars_ue;
								echo $fm_cfg['cgi'].'dir='.$fm_dir_ue.$entry_ue.'">';
								echo $entry_he2.'</a>';
							} else {
								echo $entry_he2;
							}
						} else {

							/*
							 * Setting up view URL
							 */

							if ($view_column && $fm_cfg['perm']['file']['view']) {
								$view_url = parse_url('http://'.$HTTP_SERVER_VARS['HTTP_HOST']
										.$HTTP_SERVER_VARS['REQUEST_URI']);
								$view_url['path'] = get_path_without_scriptname($view_url) 
									. (strlen($fm_cfg['url']['root']) > 0 
											? $fm_cfg['url']['root'] : $fm_cfg['dir']['root'])
									. $fm_dir_ue . $entry_ue;
								unset($view_url['query']);
								unset($view_url['fragment']);
								$view_url = str_replace('%2F', '/', glue_url($view_url));
							}

							if ($fm_cfg['show']['icons']) {
								echo '<td align=center>';
								if (is_link($entry)) {
									echo '<b>@</b>';
								} else {
									$icon_file = '';
									$icon_alt  = '';
									$ext  = substr(strtolower(strrchr($entry_he , '.')), 1);
									if (isset($fm_cfg['icons']['name'][$entry])) {
										$icon_file = $fm_cfg['icons']['name'][$entry];
										$icon_alt  = strtoupper($entry).' '.strtolower(_FM_FILE);
									} elseif (isset($fm_cfg['icons']['ext'][$ext])) {
										$icon_file = $fm_cfg['icons']['ext'][$ext];
										$icon_alt  = strtoupper($ext).' '.strtolower(_FM_FILE);
									}

									if ($icon_file == '') {
										echo '&nbsp;';
									} else {

										if ($fm_cfg['perm']['file']['view'])  {
											echo '<a target=_blank href="'.$view_url.'">';
										}

										echo '<img alt="'.$icon_alt.'" border=0 src="'
											. $fm_cfg['url']['icons'].$icon_file.'">';

										if ($fm_cfg['perm']['file']['view']) {
											echo '</a>';
										}
									}
								}

								echo '</td>';
							}
							echo '<td width="100%">' . $entry_he2;
						}

						echo '</td>';

						if (! @is_dir($entry)) {
							if ($fm_cfg['show']['size']) {
								echo '<td align=right nowrap><small>';
								$size = @filesize($entry);
								if (is_numeric($size)) {
									$unit = 'B';
									if ( $size > 1073741824 ) {
										$size = round($size / 1073741824, 1);
										$unit = 'GB';
									} elseif ( $size > 1048576 ) {
										$size = round($size / 1048576, 1);
										$unit = 'MB';
									} elseif ( $size > 1024 ) {
										$size = round($size / 1024, 1);
										$unit = 'kB';
									}
									echo $size . '&nbsp;' . $unit;
								}
								echo '</small></td>';
							}

							if ($fm_cfg['show']['date']) {
								echo '<td align=center nowrap><small>';
								echo date($fm_cfg['format']['date'], @filemtime($entry));
								echo '</small></td>';
							}

							if ($rename_column) {
								echo '<td><small>';
								if ($fm_cfg['perm']['file']['rename']) {
									echo '[<a href="'.$PHP_SELF_origvars_ue
										. $fm_cfg['cgi'].'dir='.$fm_dir_ue.'&amp;'
										. $fm_cfg['cgi'].'action=confirm_rename_file'.'&amp;'
										. $fm_cfg['cgi'].'filename='.$entry_ue.'">'
										. _FM_FILE_RENAME.'</a>]';
								}
								echo '</small></td>';
							}

							if ($delete_column) {
								echo '<td><small>';
								if ($fm_cfg['perm']['file']['delete']) {
									echo '[<a href="'.$PHP_SELF_origvars_ue
										. $fm_cfg['cgi'].'dir='.$fm_dir_ue.'&amp;'
										. $fm_cfg['cgi'].'action=confirm_delete_file'.'&amp;'
										. $fm_cfg['cgi'].'filename='.$entry_ue.'">'
										. _FM_FILE_DELETE.'</a>]';
								}
								echo '</small></td>';
							}

							if ($view_column) {
								echo '<td><small>';
								if ($fm_cfg['perm']['file']['view']) {
									echo '[<a target=_blank href="'.$view_url.'">'
										. _FM_FILE_VIEW.'</a>]';
								}
								echo '</small></td>';
							}

							if ($edit_column) {
								echo '<td><small>';
								if ($fm_cfg['perm']['file']['edit']
										&& @is_readable($entry)
										&& @is_writeable($entry)) {
									echo '[<a href="'.$PHP_SELF_origvars_ue
										. $fm_cfg['cgi'].'dir='.$fm_dir_ue.'&amp;'
										. $fm_cfg['cgi'].'action=confirm_edit_file'.'&amp;'
										. $fm_cfg['cgi'].'filename='.$entry_ue.'">'
										. _FM_FILE_EDIT.'</a>]';
								}
								echo '</small></td>';
							}
						} else {

							if ($fm_cfg['show']['size']) {
								echo '<td></td>';
							}

							if ($fm_cfg['show']['date']) {
								echo '<td></td>';
							}

							if ($rename_column) {
								echo '<td><small>';
								if ($fm_cfg['perm']['dir']['rename']) {
									echo '[<a href="'.$PHP_SELF_origvars_ue
										. $fm_cfg['cgi'].'dir='.$fm_dir_ue.'&amp;'
										. $fm_cfg['cgi'].'action=confirm_rename_directory'.'&amp;'
										. $fm_cfg['cgi'].'filename='.$entry_ue.'">'
										. _FM_DIR_RENAME.'</a>]';
								}
								echo '</small></td>';
							}

							if ($delete_column) {
								echo '<td><small>';
								if ($fm_cfg['perm']['dir']['remove']) {
									echo '[<a href="'.$PHP_SELF_origvars_ue
										. $fm_cfg['cgi'].'dir='.$fm_dir_ue.'&amp;'
										. $fm_cfg['cgi'].'action=confirm_remove_directory'.'&amp;'
										. $fm_cfg['cgi'].'filename='.$entry_ue.'">'
										. _FM_DIR_REMOVE.'</a>]';
								}
								echo '</small></td>';
							}

							if ($view_column) {
								echo '<td><small>';
								if ($fm_cfg['perm']['dir']['enter']
										&& ($fm_cfg['perm']['dir']['follow_symlinks']
											|| ! is_link($entry))) {
									echo '[<a href="'.$PHP_SELF_origvars_ue
										. $fm_cfg['cgi'].'dir='.$fm_dir_ue.$entry_ue.'">'
										. _FM_DIR_ENTER.'</a>]';
								}
								echo '</small></td>';
							}

							if ($edit_column) {
								echo '<td><small>&nbsp;</small></td>';
							}
						}

						echo '</tr>'."\n";
					}
				}

				$d->close();
			}

			if ($last_row) {
				echo '<tr><td colspan="'.$colspan.'" align=center><br><small>';

				if ($fm_cfg['perm']['dir']['create']) {
					echo '[<a href="'.$PHP_SELF_origvars_ue
						. $fm_cfg['cgi'].'dir='.$fm_dir_ue.'&amp;'
						. $fm_cfg['cgi'].'action=confirm_create_directory">'
						. _FM_DIR_CREATE.'</a>]';
					echo '&nbsp;';
				}

				if ($fm_cfg['perm']['file']['create']) {
					echo '[<a href="'.$PHP_SELF_origvars_ue
						. $fm_cfg['cgi'].'dir='.$fm_dir_ue.'&amp;'
						. $fm_cfg['cgi'].'action=confirm_create_file">'
						. _FM_FILE_CREATE.'</a>]';
					echo '&nbsp;';
				}

				if ($fm_cfg['perm']['file']['upload']) {
					echo '[<a href="'.$PHP_SELF_origvars_ue
						. $fm_cfg['cgi'].'dir='.$fm_dir_ue.'&amp;'
						. $fm_cfg['cgi'].'action=confirm_upload_file">'
						. _FM_FILE_UPLOAD.'</a>]';
					echo '&nbsp;';
				}

				echo '</small></td></tr>'."\n";
			}

			echo '</table>'."\n";

	}

	echo '</form>'."\n";

	/* Now return back to the initial working directory. */
	chdir($fm_orig_dir);

	if (defined('LOADED_AS_MODULE')) {
		CloseTable();
		include 'footer.php'; 
	}
}

/*
 * Main function call
 */

fm_main();

?>
